local has_words_before = function()
    local line, col = unpack(vim.api.nvim_win_get_cursor(0))
    return col ~= 0 and
        vim.api.nvim_buf_get_lines(0, line - 1, line, true)[1]:sub(
            col, col):match("%s") == nil
end

local cmp = require('cmp')
local luasnip = require("luasnip")

cmp.setup({
    snippet = {
        expand = function(args)
            require("luasnip").lsp_expand(args.body)
        end
    },
    sources = require("cmp").config.sources({
        { name = "nvim_lsp" }, { name = "luasnip" }, { name = "path" },
        { name = "buffer" }, { name = "git" },
        { name = 'nvim_lsp_signature_help' },
        { name = "conventionalcommits" }, { name = "nvim-lua" },
        { name = "calc" }
    }),
    mapping = {
        ["<C-Space>"] = cmp.mapping(cmp.mapping.complete(),
            { "i", "c" }),
        ["<CR>"] = cmp.mapping(function(fallback)
            if cmp.visible() and cmp.get_selected_entry() then
                cmp.confirm()
            else
                fallback()
            end
        end, { "i", "s" }),
        ["<Tab>"] = cmp.mapping(function(fallback)
            if cmp.visible() then
                cmp.select_next_item()
            elseif luasnip.expand_or_jumpable() then
                luasnip.expand_or_jump()
            elseif has_words_before() then
                cmp.complete()
            else
                fallback()
            end
        end, { "i", "s" }),

        ["<S-Tab>"] = cmp.mapping(function(fallback)
            if cmp.visible() then
                cmp.select_prev_item()
            elseif luasnip.jumpable(-1) then
                luasnip.jump(-1)
            else
                fallback()
            end
        end, { "i", "s" })
    },
    experimental = { ghost_text = true }
})

-- `/` cmdline setup.
cmp.setup.cmdline('/', { sources = { { name = 'buffer' } } })

-- `:` cmdline setup.
cmp.setup.cmdline(':', {
    sources = cmp.config
        .sources({ { name = 'path' } }, { { name = 'cmdline' } })
})
